subroutine labeling2(indata,label,NX,NY,iarea,LABELMAX,number_of_label)
implicit none
integer, parameter :: LABEL_MAX=10000
integer, parameter :: DP = kind(1.0d0)
integer, parameter :: SP = kind(1.0)
integer, parameter :: DPC = kind((1.0d0,1.0d0))
real(DP), parameter :: PI=3.14159265358979323846D0
real(DP), parameter :: PI4=PI*4
!
integer,intent(in) :: NX,NY
integer,intent(in) :: LABELMAX
integer,dimension(0:NX,0:NY),intent(in) :: indata
integer,dimension(0:NX,0:NY),intent(out) :: label
integer,intent(out) :: number_of_label
integer,dimension(LABELMAX),intent(out) :: iarea
!
integer :: i,j,l,l0,l1,current_label
integer,dimension(0:LABEL_MAX) :: lookup,lookup1
integer,dimension(LABEL_MAX) :: rank,order,iarea_work
!
if(LABELMAX.GT.LABEL_MAX) then
 write(*,*) '**WARNING(labeling2): LABELMAX > LABEL_MAX: ',LABELMAX,LABEL_MAX
end if
!
label(:,:) = 0
lookup(:) = 0
current_label=0
!
do j = 0, NY
do i = 0, NX
  if(indata(i,j).EQ.1) then
    if(i.eq.0.AND.j.eq.0) then
         current_label=current_label+1 
         label(i,j) = current_label
         lookup(current_label)=current_label
     else if(j.eq.0) then
         if(label(i-1,j).NE.0) then
           label(i,j) = label(i-1,j)
         else
           current_label=current_label+1
           label(i,j) = current_label
           lookup(current_label)=current_label
         end if
     else if(i.eq.0) then
         if(label(i,j-1).NE.0) then
           label(i,j) = label(i,j-1)
         else
           current_label=current_label+1
           label(i,j) = current_label
           lookup(current_label)=current_label
         end if
      else
         if(label(i-1,j).ne.0.AND.label(i,j-1).ne.0) then
            label(i,j) = min(label(i-1,j), &
                             label(i,j-1))
            if(label(i-1,j).ne.label(i,j-1)) then
               lookup(max(label(i-1,j),label(i,j-1)))=label(i,j)
            end if
         else if(label(i-1,j).ne.0) then
            label(i,j) = label(i-1,j)
         else if(label(i,j-1).ne.0) then
            label(i,j) = label(i,j-1)
         else
            current_label=current_label+1
            label(i,j) = current_label
            lookup(current_label)=current_label
         end if
    end if
  end if
end do
end do
!
if(current_label.eq.0) then
    write(*,*) 'LABELING: no connected_component found.'
    return
end if
!
do l = 1, current_label
   l0=l
   do while (l0.NE.lookup(l0))
     l0=lookup(l0)
   end do
   lookup(l) = l0
end do
!
l0=0
l1=0
do l = 1, current_label
   if(lookup(l).GT.l0) then
      l0=max(l0,lookup(l))
      l1=l1+1
      lookup1(l)=l1
    else
      lookup1(l)=lookup1(lookup(l))
   end if
end do
number_of_label = l1

do i = 0, NX
do j = 0, NY
   label(i,j) = lookup1(lookup(label(i,j)))
end do
end do
 
if(number_of_label.GT.LABELMAX) then
 write(*,*) '**WARNING: number_of_label > LABELMAX: ',number_of_label,LABELMAX
 write(*,*) '** Labeling number is not ordered.'
 return
end if

iarea_work(:)=0
do i = 0, NX
do j = 0, NY
  if(label(i,j).NE.0) then
     iarea_work(label(i,j))=iarea_work(label(i,j))+1
  end if
end do
end do
!
call make_order2_i(iarea_work,rank,order,number_of_label)
!
do i = 0, NX
do j = 0, NY
   if(label(i,j).NE.0) then
      label(i,j)=rank(label(i,j))
   end if
end do
end do

do l = 1, number_of_label
 iarea(rank(l)) = iarea_work(l)
end do
!
return
end subroutine labeling2
subroutine make_order2_d(datain,rank,order,length)
! 20180518 Kanya kusano
! datain(i)がj番目の大きさの場合、
! rank(i)=j, order(j)=1
! を与える。
implicit none
real(8),dimension(length),intent(in) :: datain
real(8) :: valmax
integer,dimension(length),intent(out) :: order,rank
integer,intent(in) :: length
integer :: i,j
logical,allocatable,dimension(:) :: mask
!
allocate(mask(length))
mask(:) = .TRUE.
order(:)=0
!
do i = 1, length
  order(i) = maxloc(datain(:),1,mask(:))
  mask(order(i))=.FALSE.
end do
!
do i = 1, length
  rank(order(i))=i
end do
!
deallocate(mask)
end subroutine make_order2_d
subroutine make_order2_i(idatain,rank,order,length)
! 20180518 Kanya kusano
! idatain(i)がj番目の大きさの場合、
! rank(i)=j, order(j)=1
! を与える。
implicit none
integer,dimension(length),intent(in) :: idatain
real(8) :: valmax
integer,dimension(length),intent(out) :: order,rank
integer,intent(in) :: length
integer :: i,j
logical,allocatable,dimension(:) :: mask
!
allocate(mask(length))
mask(:) = .TRUE.
order(:)=0
!
do i = 1, length
  order(i) = maxloc(idatain(:),1,mask(:))
  mask(order(i))=.FALSE.
end do
!
do i = 1, length
  rank(order(i))=i
end do
!
deallocate(mask)
end subroutine make_order2_i
