module pgplot
implicit none
! === precision type ===
integer, parameter :: DP = kind(1.0d0)
integer, parameter :: SP = kind(1.0)
integer, parameter :: DPC = kind((1.0d0,1.0d0))
!
real(SP),dimension(:,:),allocatable :: F_s,FX_s,FY_s
real(SP) :: fmin, fmax, TR(6)
DATA TR/0.,1.,0.,0.,0.,1./
public :: pgplot__set, pgplot__image2, pgplot__contour, &
          pgplot__vector, pgplot__end
!private ::
!
interface pgplot__image2
     module procedure pgplot__image_s
     module procedure pgplot__image_d
end interface pgplot__image2
interface pgplot__contour
     module procedure pgplot__contour_s
     module procedure pgplot__contour_d
end interface pgplot__contour
interface pgplot__vector
     module procedure pgplot__vector_s
     module procedure pgplot__vector_d
end interface pgplot__vector
!!!!!!!!!
contains
subroutine pgplot__set(width,aspect,x0,y0,delx,dely,N_x,N_y,text,device,same_scale,log_scale)
real(4),intent(in) :: x0,y0,delx,dely
real(4) :: x1,y1
real(4) :: width,aspect
integer,intent(in) :: N_x, N_y
character(*),intent(in) :: text,device
integer :: pgbeg
logical,intent(in) :: same_scale,log_scale(2)

TR(1) = x0-delx
TR(2) = delx
TR(4) = y0-dely
TR(6) = dely
x1=x0+delx*(N_x-1)
y1=y0+dely*(N_y-1)

IF (PGBEG(0,device,1,1) .NE. 1) STOP
! IF (PGBEG(0,'?',1,1) .NE. 1) STOP
! IF (PGBEG(0,'/xwin',1,1) .NE. 1) STOP
      call pgpap(width,aspect)
      CALL PGPAGE
      CALL PGSVP(0.06,0.97,0.15,0.97)
   if(same_scale) then
      call pgwnad(x0,x1,y0,y1) 
   else
      CALL PGSWIN(x0,x1,y0,y1)
   end if
     if(log_scale(1).AND.log_scale(2)) then
      CALL PGBOX('bclntsi',0.0,0,'bclntsiv',0.0,0)
     else if(log_scale(1).AND..NOT.log_scale(2)) then
!      call PGENV(x0,x1,y0,y1,0,10)
      CALL PGBOX('bcglntsi',0.0,0,'bcgntsiv',0.0,0)
     else if(.NOT.log_scale(1).AND.log_scale(2)) then
      CALL PGBOX('bcntsi',0.0,0,'bclntsiv',0.0,0)
     else
      CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
     end if
      CALL PGMTXT('b',3.0,1.0,1.0,trim(text))
      CALL PGBBUF
end subroutine pgplot__set
subroutine pgplot__end
      CALL PGEBUF
      call PGEND
end subroutine pgplot__end
subroutine pgplot__image_d(Fd,N_x,N_y,dmin,dmax,ipalett)
integer,intent(in) :: N_x, N_y, ipalett
real(DP),intent(in) :: dmin, dmax
real(DP),dimension(N_x,N_y),intent(in) :: Fd
real(SP),allocatable,dimension(:,:) :: F_s
real(SP) :: bright, contra

allocate(F_s(N_x,N_y))

F_s(:,:) = Fd(:,:)

  if(dmin.eq.0..and.dmax.eq.0.) then
      FMIN = minval(F_s)
      FMAX = maxval(F_s)
     else
      FMIN = dmin
      FMAX = dmax
  end if

      !!! write(*,*) '## fmin fmax (image) ',fmin,fmax
      if(fmin==fmax) return

      BRIGHT = 0.5
      CONTRA  = 1.0
      CALL PALETT(ipalett, CONTRA, BRIGHT)
      CALL PGIMAG(F_s,N_x,N_y,1,N_x,1,N_y,FMIN,FMAX,TR)
      CALL PGWEDG('BI', 4.0, 3.5, FMIN, FMAX, ' ')
      CALL PGSCH(1.0)

deallocate(F_s)

end subroutine pgplot__image_d
subroutine pgplot__image_s(Fs,N_x,N_y,dmin,dmax,ipalett)
integer,intent(in) :: N_x, N_y,ipalett
real(DP),intent(in) :: dmin, dmax
real(SP),dimension(N_x,N_y),intent(in) :: Fs
real(SP) :: bright, contra

  if(dmin.eq.0..and.dmax.eq.0.) then
      FMIN = minval(Fs)
      FMAX = maxval(Fs)
     else
      FMIN = dmin
      FMAX = dmax
  end if

      write(*,*) '## fmin fmax (image) ',fmin,fmax
      if(fmin==fmax) return

      BRIGHT = 0.5
      CONTRA  = 1.0
      CALL PALETT(ipalett, CONTRA, BRIGHT)
      CALL PGIMAG(Fs,N_x,N_y,1,N_x,1,N_y,FMIN,FMAX,TR)
      CALL PGWEDG('BI', 4.0, 3.5, FMIN, FMAX, ' ')
      CALL PGSCH(1.0)

end subroutine pgplot__image_s
subroutine pgplot__contour_d(Fd,N_x,N_y,dmin,del,nlev)
integer,intent(in) :: N_x, N_y, nlev
real(DP),intent(in) :: dmin, del
real(DP) :: delcont
real(DP),dimension(N_x,N_y),intent(in) :: Fd
real(SP),dimension(NLEV) :: alev
integer :: i
integer :: pgindex_lw,pgindex_ls,pgindex_ci


allocate(F_s(N_x,N_y))

F_s(:,:) = Fd(:,:)
delcont= del
FMIN = dmin

  if(delcont.eq.0.0d0.and.NLEV.gt.1) then
      FMIN = minval(F_s)
      FMAX = maxval(F_s)
      delcont = (FMAX-FMIN)/(NLEV-1)
  end if

      DO 30 I=1,NLEV
          ALEV(I) = FMIN + (I-1)*delcont
   30 CONTINUE

      CALL PGCONT(F_s,N_x,N_y,1,N_x,1,N_y,ALEV,-NLEV,TR)

deallocate(F_s)

end subroutine pgplot__contour_d
subroutine pgplot__contour_s(Fs,N_x,N_y,dmin,dmax,nlev)
integer,intent(in) :: N_x, N_y,NLEV
real(DP),intent(in) :: dmin, dmax
real(SP),dimension(N_x,N_y),intent(in) :: Fs
real(SP),dimension(NLEV) :: alev
integer :: i

  if(dmin.eq.0..and.dmax.eq.0.) then
      FMIN = minval(Fs)
      FMAX = maxval(Fs)
     else
      FMIN = dmin
      FMAX = dmax
  end if

      DO 30 I=1,NLEV
          ALEV(I) = FMIN + (I-1)*(FMAX-FMIN)/(NLEV-1)
   30 CONTINUE

      CALL PGCONT(Fs,N_x,N_y,1,N_x,1,N_y,ALEV,NLEV,TR)

      CALL PGSLW(1)
      CALL PGSLS(1)
      CALL PGSCI(1)

end subroutine pgplot__contour_s
!
SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
      INTEGER TYPE
      REAL CONTRA, BRIGHT
!
      REAL GL(2), GR(2), GG(2), GB(2)
      REAL RL(9), RR(9), RG(9), RB(9)
      REAL RL2(9), RR2(9), RG2(9), RB2(9)
      REAL HL(5), HR(5), HG(5), HB(5)
      REAL WL(10), WR(10), WG(10), WB(10)
      REAL AL(20), AR(20), AG(20), AB(20)
!
      DATA GL /0.0, 1.0/
      DATA GR /0.0, 1.0/
      DATA GG /0.0, 1.0/
      DATA GB /0.0, 1.0/
!
      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!
      DATA RL2 / 0.0, 0.1, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
      DATA RR2 / 1.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
      DATA RG2 / 1.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
      DATA RB2 / 1.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!
      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
!
      DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
      DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
      DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
      DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/
!
      DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5, &
               0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
      DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, &
               0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
      DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, &
               0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
      DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, &
               0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/

      IF (TYPE.EQ.1) THEN
!        -- gray scale
         CALL PGCTAB(GL, GR, GG, GB, 2, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.2) THEN
!        -- rainbow
         CALL PGCTAB(RL, RR, RG, RB, 9, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.22) THEN
!        -- rainbow
         CALL PGCTAB(RL2, RR2, RG2, RB2, 9, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.3) THEN
!        -- heat
         CALL PGCTAB(HL, HR, HG, HB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.4) THEN
!        -- weird IRAF
         CALL PGCTAB(WL, WR, WG, WB, 10, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.5) THEN
!        -- AIPS
         CALL PGCTAB(AL, AR, AG, AB, 20, CONTRA, BRIGHT)
      END IF
END subroutine PALETT
! ======================================
subroutine pgplot__vector_s(fx_ss,fy_ss,N_x,N_y,skip_x,skip_y,factor_d)
! ======================================
integer,intent(in) :: N_x,N_y
real(SP),dimension(N_x,N_y),intent(in) :: fx_ss, fy_ss
integer :: skip_x, skip_y
real(DP),intent(in) :: factor_d
real(SP) :: factor
real(SP) :: tr_vec(6)
integer :: np_x, np_y, i, j, ii, jj

factor = factor_d

np_x = 1+(N_x-1)/skip_x
np_y = 1+(N_y-1)/skip_y

allocate(FX_s(np_x,np_y))
allocate(FY_s(np_x,np_y))

!!$tr_vec(1) = 1.0-skip_x
!!$tr_vec(2) = skip_x
!!$tr_vec(3) = 0.0
!!$tr_vec(4) = 1.0-skip_y
!!$tr_vec(5) = 0.0
!!$tr_vec(6) = skip_y

tr_vec(1) = tr(1)+tr(2)*(1-skip_x)
tr_vec(2) = tr(2)*skip_x
tr_vec(3) = tr(3)
tr_vec(4) = tr(4)+tr(6)*(1-skip_y)
tr_vec(5) = tr(5)
tr_vec(6) = tr(6)*skip_y

do i = 1, np_x
  ii = 1 + (i-1)*skip_x
do j = 1, np_y
  jj = 1 + (j-1)*skip_y
  FX_s(i,j) = fx_ss(ii,jj)
  FY_s(i,j) = fy_ss(ii,jj)
end do
end do

call PGVECT(FX_s, FY_s, np_x, np_y, 1, np_x, 1, np_y, factor, &
            1, tr_vec, 0.0)

deallocate(FX_s)
deallocate(FY_s)

end subroutine pgplot__vector_s
! ======================================
subroutine pgplot__vector_d(fx_d,fy_d,N_x,N_y,skip_x,skip_y,factor_d)
integer,intent(in) :: N_x,N_y
real(DP),dimension(N_x,N_y),intent(in) :: fx_d, fy_d
integer :: skip_x, skip_y
real(DP),intent(in) :: factor_d
real(SP) :: factor
real(SP) :: tr_vec(6)
integer :: np_x, np_y, i, j, ii, jj

factor = factor_d

np_x = 1+(N_x-1)/skip_x
np_y = 1+(N_y-1)/skip_y

allocate(FX_s(np_x,np_y))
allocate(FY_s(np_x,np_y))

!!$tr_vec(1) = 1.0-skip_x
!!$tr_vec(2) = skip_x
!!$tr_vec(3) = 0.0
!!$tr_vec(4) = 1.0-skip_y
!!$tr_vec(5) = 0.0
!!$tr_vec(6) = skip_y

tr_vec(1) = tr(1)+tr(2)*(1-skip_x)
tr_vec(2) = tr(2)*skip_x
tr_vec(3) = tr(3)
tr_vec(4) = tr(4)+tr(6)*(1-skip_y)
tr_vec(5) = tr(5)
tr_vec(6) = tr(6)*skip_y

do i = 1, np_x
  ii = 1 + (i-1)*skip_x
do j = 1, np_y
  jj = 1 + (j-1)*skip_y
  FX_s(i,j) = fx_d(ii,jj)
  FY_s(i,j) = fy_d(ii,jj)
end do
end do

call PGVECT(FX_s, FY_s, np_x, np_y, 1, np_x, 1, np_y, factor, &
            1, tr_vec, 0.0)

deallocate(FX_s)
deallocate(FY_s)

end subroutine pgplot__vector_d
end module pgplot
