!########### Spline interpolatin for 2D array ###################
!################# periodic data version ########################
subroutine spline_2d_periodic(sample2d,target2d,x_for_sample,y_for_sample, &
                                                x_for_target,y_for_target, &
                                                NSX,NSY,NTX,NTY)
!................................................................
!Made by K.Kusano (The Earth Simulator Center)
!modified on 20070119
!
!  This module is for the spline interpolation of 2d data, in which
!  the periodic boundary condition is imposed onto both the x and y
!  coordinates, and the grid interval is uniform both for the sample
!  and target data.
!
! Note:
!  sample2d(1,:) is defined at x = 0, and
!  sample2d(NSX,:) is defined at x = p*(NSX-1)/NSX
!  (not at x = p), where p is the period.
!
! target2d is defined also in the same way, so that
! the position where sample2d(NSX,:) is not defined at
! the position where target2d(NTX,:) is defined, when NTX.ne.NSX,
! even in case of uniform grids. 
!................................................................
!
implicit none 
real(8),dimension(NSX,NSY),intent(in)  :: sample2d
real(8),dimension(NTX,NTY),intent(out) :: target2d
real(8),intent(in) :: x_for_sample(NSX), y_for_sample(NSY), &
                      x_for_target(NTX), y_for_target(NTY)
integer, intent(in) :: NSX, NSY, NTX, NTY
!
real(8),dimension(:,:),allocatable :: value_on_yaxis
real(8),dimension(:),allocatable :: sample_x
real(8),dimension(:),allocatable :: sample_y
real(8),dimension(:),allocatable :: x_for_sample1
real(8),dimension(:),allocatable :: y_for_sample1
!real(8),dimension(:),allocatable :: x_for_target
!real(8),dimension(:),allocatable :: y_for_target
real(8) :: delx_sample,dely_sample,delx_target,dely_target
integer :: i, j
real(8) :: xl = 1.0d0, yl=1.0d0 
!
!#### allocata arrays ####
!
allocate(value_on_yaxis(NSX,NTY))
allocate(sample_x(NSX+1))
allocate(sample_y(NSY+1))
allocate(x_for_sample1(NSX+1))
allocate(y_for_sample1(NSY+1))
! allocate(x_for_target(NTX))
! allocate(y_for_target(NTY))

!#### set coordinate ####
!
!!$delx_sample = xl/NSX
!!$dely_sample = yl/NSY
!!$delx_target = xl/NTY
!!$dely_target = yl/NTY
!!$
!!$do i = 1, NSX+1
!!$   x_for_sample(i) = delx_sample*(i-1)
!!$end do
!!$!
!!$do j = 1, NSY+1
!!$   y_for_sample(j) = dely_sample*(j-1)
!!$end do
!!$!
!!$do i = 1, NTX
!!$   x_for_target(i) = delx_target*(i-1)
!!$end do
!!$!
!!$do j = 1, NTY
!!$   y_for_target(j) = dely_target*(j-1)
!!$end do

!#### interpolation ####

x_for_sample1(1:NSX) = x_for_sample(1:NSX)
x_for_sample1(NSX+1) = x_for_sample(NSX)*2 - x_for_sample(NSX-1)
y_for_sample1(1:NSY) = y_for_sample(1:NSY)
y_for_sample1(NSY+1) = y_for_sample(NSY)*2 - y_for_sample(NSY-1)

!!$!#DEBUG---
!!$open(50,file='TEST_XS',form='FORMATTED')
!!$do i = 1, NSX+1
!!$   write(50,*) i,x_for_sample1(i)
!!$end do
!!$close(50)
!!$
!!$open(50,file='TEST_YS',form='FORMATTED')
!!$do j = 1, NSY+1
!!$   write(50,*) j,y_for_sample1(j)
!!$end do
!!$close(50)
!!$
!! open(50,file='TEST',form='FORMATTED')
!! do j = 1, NSY+1
!!    write(50,*) j,y_for_sample1(j),sample2d(NSX/2,j)
!! end do
!! close(50)
!!$
!!$open(50,file='TEST_XT',form='FORMATTED')
!!$do i = 1, NTX
!!$   write(50,*) i,x_for_target(i)
!!$end do
!!$close(50)
!!$
!!$open(50,file='TEST_YT',form='FORMATTED')
!!$do j = 1, NTY
!!$   write(50,*) j,y_for_target(j)
!!$end do
!!$close(50)
!!$
!!$!---------

do i = 1, NSX
  do j = 1, NSY
   sample_y(j)=sample2d(i,j)
  end do

   sample_y(NSY+1)=sample_y(1)
!
   call spline_1d(y_for_sample1,NSY+1,sample_y, &
                  y_for_target,NTY,  value_on_yaxis(i,1:NTY))
end do
!
do j = 1, NTY
   sample_x(1:NSX)=value_on_yaxis(1:NSX,j)
   sample_x(NSX+1)=sample_x(1)

   call spline_1d(x_for_sample1,NSX+1,sample_x, &
                  x_for_target(1:NTX),NTX,target2d(1:NTX,j))
end do
!
!#### allocata arrays ####
!
deallocate(value_on_yaxis)
deallocate(sample_x)
deallocate(sample_y)
deallocate(x_for_sample1)
deallocate(y_for_sample1)
! deallocate(x_for_target)
! deallocate(y_for_target)

end subroutine spline_2d_periodic
!##################### 1D version ######################
subroutine spline_1d(x_sample,N_sample,v_sample, &
                     x_target,N_target,v_target)
real(8),dimension(N_sample),intent(in)  :: x_sample, v_sample
real(8),dimension(N_target),intent(out) :: x_target, v_target
integer, intent(in) :: N_sample, N_target
real(8),dimension(:,:),allocatable :: KEISU
real(8),dimension(2) :: DF
integer,dimension(2) :: IOPT
integer :: NC, ier
!
NC = N_sample - 1
IOPT(1) = 3 ! no information at x(1)
IOPT(2) = 3 ! no information at x(N_sample)

allocate(KEISU(NC,3))

call splc(x_sample,N_sample,v_sample, DF, IOPT, &
          KEISU, NC, ier)
if(ier.ne.0) then
   write(*,*) '## WARNING: splc ERR ',ier
end if

call splf(x_sample,N_sample,v_sample,KEISU,NC, &
          x_target,N_target,v_target,ier)
if(ier.ne.0) then
   write(*,*) '## WARNING: splf ERR ',ier
end if

deallocate(KEISU)

end subroutine spline_1d


